﻿using System.Security.Cryptography.X509Certificates;
using System.Text;
using Infrastructure.Utils.Xml;
using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Service.Proxy.ResourceItems;
using OrganisatieTypeV1 = Contracts.Generated.Flows.Schuldbemiddeling.v1.OrganisatieType;
using OrganisatieTypeV2 = Contracts.Generated.Systeem.v2.OrganisatieType;

namespace SoapEndpoint.Tester.Infrastructure;

public static class FormHelperMethods
{
    public static Guid ParseGuid(string input) => string.IsNullOrEmpty(input) ? Guid.Empty : Guid.Parse(input);

    public static void SetGuid(TextBox tb)
    {
        tb.Text = Guid.NewGuid().ToString();
    }

    public static Guid? ParseOptionalGuid(string value) =>
        string.IsNullOrWhiteSpace(value) ? default(Guid?) : Guid.Parse(value);

    public static string GetOptionalValueAsString(string value) => string.IsNullOrWhiteSpace(value) ? null : value;

    public static void BindData<T>(
        DataGridView dgv,
        IList<T> items, Func<T, object[]> converter) where T : class
    {
        dgv.Rows.Clear();

        foreach (var item in items)
        {
            var index = dgv.Rows.Add(converter(item));

            dgv.Rows[index].Tag = item;
        }
    }

    public static T GetMessageBody<T>(string xml)
    {
        try
        {
            var obj = DataContractSerializeHelper.Deserialize<T>(xml);
            return obj;
        }
        catch (Exception)
        {
            return default;
        }
    }

    private static List<string> EnumStrings<T>() where T : Enum
    {
        var types = Enum.GetValues(typeof(T))
            .Cast<T>()
            .Select(t => t.ToString());

        // Include an 'all' option in the possible values
        var values = new List<string> { "Alle" };
        values.AddRange(types);
        return values;
    }

    public static void BindFlows(ComboBox cb, FlowType? defaultFlowType = null, bool includeAllOption = false)
    {
        if (includeAllOption)
        {
            cb.DataSource = EnumStrings<FlowType>();
        }
        else
        {
            cb.DataSource = Enum.GetValues(typeof(FlowType));
        }

        if (defaultFlowType == null)
        {
            return;
        }

        if (includeAllOption)
        {
            cb.SelectedItem = defaultFlowType.Value.ToString();
        }
        else
        {
            cb.SelectedItem = defaultFlowType.Value;
        }
    }

    public static void BindVersions(ComboBox cb, Version[] versions, Version defaultVersion = default)
    {
        cb.DataSource = versions.Clone();
        cb.DisplayMember = "Name";

        if (defaultVersion != null)
        {
            cb.SelectedItem = defaultVersion;
        }
    }

    public static void BindVersions(ListBox cb, Version[] versions)
    {
        cb.DataSource = versions.Clone();
        cb.DisplayMember = "Name";
    }

    public static void BindMessageTypes<TMessageType>(ComboBox cb, ResourceItem<TMessageType>[] resourceItems)
        where TMessageType : Enum
    {
        cb.DataSource = resourceItems.Clone();
        cb.DisplayMember = "Name";
        if (resourceItems.Length > 0)
        {
            cb.SelectedItem = resourceItems[0];
        }
    }

    public static void BindEnvironments(ComboBox cb, Environment environment)
    {
        cb.DataSource = Environments.Items;
        cb.DisplayMember = "Name";
        if (environment != null)
        {
            cb.SelectedItem = environment;
        }
    }

    public static void BindMessageTypes<TMessageType>(ComboBox cb) where TMessageType : Enum
    {
        cb.DataSource = EnumStrings<TMessageType>();
    }

    public static void BindOrganizationTypes(ComboBox cb, Version systemVersion)
    {
        switch (systemVersion.Major)
        {
            case 1:
                cb.DataSource = EnumStrings<OrganisatieTypeV1>();
                break;
            case 2:
                cb.DataSource = EnumStrings<OrganisatieTypeV2>();
                break;
            default:
                throw new InvalidOperationException($"Unknown system version: {systemVersion.Major}");
        }
    }

    public static void BindCertificate(ComboBox cb, IList<CertificateItem> items, string emptyItemText = null)
    {
        if (!string.IsNullOrWhiteSpace(emptyItemText))
        {
            items.Insert(0, new CertificateItem(emptyItemText));
        }

        cb.DataSource = items.ToArray();
        cb.DisplayMember = "Name";
    }

    public static X509Certificate2 GetCertificate(ComboBox combobox)
    {
        var item = combobox.SelectedItem as CertificateItem;
        return item?.Certificate;
    }

    public static T GetSelectedValue<T>(ComboBox combobox) where T : class
    {
        var item = combobox.SelectedItem as T;
        return item;
    }

    public static T GetSelectedItem<T>(DataGridView dgv) where T : class
    {
        if (dgv.CurrentCell == null)
        {
            return default;
        }

        var item = dgv.Rows[dgv.CurrentCell.RowIndex]?.Tag as T;
        return item;
    }

    public static bool ExecuteAction(Action action, Action onSuccess, Action<Exception> onError)
    {
        try
        {
            action();
            onSuccess();
            return true;
        }
        catch (Exception ex)
        {
            onError(ex);
        }

        return false;
    }

    public static async Task<bool> ExecuteActionAsync(Func<Task> action, Action onSuccess,
        Action<Exception> onError)
    {
        try
        {
            await action();
            onSuccess();
            return true;
        }
        catch (Exception ex)
        {
            onError(ex);
        }

        return false;
    }

    public static void AddColoredMessage(RichTextBox rtb, string message, bool success = true)
    {
        void SetColoredText(string text)
        {
            if (message != null)
            {
                var color = success ? Color.Black : Color.DarkRed;
                rtb.SelectionStart = rtb.TextLength;
                rtb.SelectionLength = 0;
                rtb.SelectionColor = color;
                rtb.AppendText(text);
                rtb.SelectionColor = rtb.ForeColor;
                rtb.AppendText(System.Environment.NewLine);
                rtb.SelectionStart = rtb.Text.Length;
                rtb.ScrollToCaret();
            }
        }

        if (rtb.InvokeRequired)
        {
            rtb.Invoke((MethodInvoker)delegate { SetColoredText(message); }, null);
        }
        else
        {
            SetColoredText(message);
        }
    }

    public static void SaveAttachment(BijlageModel bijlage)
    {
        var extension = bijlage.BerichtExtensieType.ToString().ToLower();

        using var sfd = new SaveFileDialog();
        sfd.FileName = bijlage.Bestandsnaam;
        sfd.Filter = $@"{extension} | *.{extension}";

        if (sfd.ShowDialog() == DialogResult.Cancel)
        {
            return;
        }

        var fileName = sfd.FileName.EndsWith($".{extension}") ? sfd.FileName : sfd.FileName + $".{extension}";

        File.WriteAllBytes(Path.GetFullPath(fileName), bijlage.Data);
    }

    public static void SetAttachments(List<BijlageModel> bijlages, Action<string, bool> message)
    {
        var openFileDialog = new OpenFileDialog();

        openFileDialog.Multiselect = true;

        if (openFileDialog.ShowDialog() == DialogResult.Cancel)
        {
            return;
        }

        if (!openFileDialog.FileNames.Any())
        {
            return;
        }

        foreach (var pathToFile in openFileDialog.FileNames)
        {
            var fileAsByteArray = File.ReadAllBytes(pathToFile);

            var fileAsBase64 = Convert.ToBase64String(fileAsByteArray);

            var extension = Path.GetExtension(pathToFile).Replace(".", "");

            var fileName = Path.GetFileName(pathToFile);

            var canParse = Enum.TryParse(extension, true, out BerichtExtensieType contentType);

            if (canParse)
            {
                message($"Added the following file {pathToFile}", true);

                bijlages.Add(new BijlageModel
                {
                    Data = Encoding.ASCII.GetBytes(fileAsBase64),
                    BerichtExtensieType = contentType,
                    Bestandsnaam = fileName
                });
                continue;
            }

            message(
                $"Unsupported file extension: {extension} for the file: {pathToFile}. Please upload your files again",
                false);

            bijlages.Clear();

            return;
        }
    }
}