﻿using System.Security.Cryptography.X509Certificates;
using AutoMapper;
using SoapEndpoint.Service.Proxy.Infrastructure;
using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.ServiceProxies.Flows;
using SoapEndpoint.Service.Proxy.ServiceProxies.Systeem;

namespace SoapEndpoint.Tester.Infrastructure;

public class ServiceUrlInfo
{
    public const string
        ConnectionTestService = "ConnectionTestService",
        ShvService = "ShvService",
        SeService = "SeService",
        MessageBoxService = "MessageBoxService",
        AddressService = "AddressService",
        LogbookService = "LogbookService";

    private readonly ServiceProxyResolver _proxyResolver;

    public ServiceUrlInfo(IMapper mapper)
    {
        _proxyResolver = new ServiceProxyResolver(mapper);
    }

    public string LogKey { get; set; }

    public IMessageBoxProxy GetMessageBoxProxy(X509Certificate2 certificate, Environment environment,
        Version version)
    {
        var url = GetSystemUrl(MessageBoxService, environment, version);
        return _proxyResolver.GetMessageBoxProxy(certificate, version, url);
    }

    public IAddressBookProxy GetAddressProxy(X509Certificate2 certificate, Environment environment,
        Version version)
    {
        var url = GetSystemUrl(AddressService, environment, version);
        return _proxyResolver.GetAddressProxy(certificate, version, url);
    }

    public ILogbookProxy<TServiceMessageType> GetLogbookProxy<TServiceMessageType>(X509Certificate2 certificate,
        Environment environment, Version version, Version flowVersion, FlowType flowType)
        where TServiceMessageType : Enum
    {
        var url = version.Major == 1
            ? GetSystemUrl(LogbookService, environment, version)
            : GetLogbookUrl(flowType, environment, flowVersion);
        return _proxyResolver.GetLogbookProxy<TServiceMessageType>(certificate, version, flowType, url);
    }

    public IMessageProxy<TServiceMessageType> GetShvProxy<TServiceMessageType>(X509Certificate2 certificate,
        FlowType flowType,
        Environment environment, Version flowVersion) where TServiceMessageType : Enum
    {
        var url = GetShvUrl(flowType, environment, flowVersion);
        return _proxyResolver.GetShvProxy<TServiceMessageType>(certificate, flowType, flowVersion, url);
    }

    public IMessageProxy<TServiceMessageType> GetSeProxy<TServiceMessageType>(X509Certificate2 certificate,
        FlowType flowType,
        Environment environment, Version flowVersion) where TServiceMessageType : Enum
    {
        var url = GetSeUrl(flowType, environment, flowVersion);
        return _proxyResolver.GetSeProxy<TServiceMessageType>(certificate, flowType, flowVersion, url);
    }

    public IConnectionTestProxy GetConnectionTestProxy(X509Certificate2 certificate, Environment environment,
        Version version)
    {
        var url = GetSystemUrl(ConnectionTestService, environment, version);
        return _proxyResolver.GetConnectionTestProxy(certificate, version, url);
    }

    public string GetSystemUrl(string service, Environment environment, Version systemVersion)
    {
        string result;
        if (systemVersion.Major == SystemVersions.V1_0.Major)
        {
            result = $"{environment?.BaseAddress}/{service}/v{systemVersion.Major}";
        }
        else if (systemVersion.Major == SystemVersions.V2_0.Major)
        {
            result = $"{environment?.BaseAddress}/systeem/{service}/v{systemVersion.Major}";
        }
        else
        {
            result = string.Empty;
        }

        if (!string.IsNullOrEmpty(LogKey))
        {
            result += $"?key={LogKey}";
        }

        return result;
    }

    public string GetShvUrl(FlowType flowType, Environment environment, Version flowVersion)
    {
        var url = flowType switch
        {
            FlowType.Schuldbemiddeling => $"{environment?.BaseAddress}/{ShvService}/v{flowVersion.Major}",
            FlowType.SbEnSk => $"{environment?.BaseAddress}/flows/sb_sk/{ShvService}/v{flowVersion.Major}",
            FlowType.Waarborgfonds =>
                $"{environment?.BaseAddress}/flows/waarborgfonds/{ShvService}/v{flowVersion.Major}",
            FlowType.Inventarisatie =>
                $"{environment?.BaseAddress}/flows/inventarisatie/{ShvService}/v{flowVersion.Major}",
            FlowType.BrEnHf => $"{environment?.BaseAddress}/flows/br_hf/{ShvService}/v{flowVersion.Major}",
            FlowType.Bewindvoering =>
                $"{environment?.BaseAddress}/flows/bewindvoering/{ShvService}/v{flowVersion.Major}",
            FlowType.PauzeSignaal3 =>
                $"{environment?.BaseAddress}/flows/pauze_signaal_3/{ShvService}/v{flowVersion.Major}",
            FlowType.ShvSignaal => $"{environment?.BaseAddress}/flows/shv_signaal/{ShvService}/v{flowVersion.Major}",
            _ => throw new InvalidOperationException(
                $"No service URL found for the selected flowtype: {flowType} and Version: {flowVersion}")
        };

        return !string.IsNullOrEmpty(LogKey) ? url + $"?key={LogKey}" : url;
    }

    private string GetSeUrl(FlowType flowType, Environment environment, Version flowVersion)
    {
        var url = flowType switch
        {
            FlowType.Schuldbemiddeling => $"{environment?.BaseAddress}/{SeService}/v{flowVersion.Major}",
            FlowType.SbEnSk => $"{environment?.BaseAddress}/flows/sb_sk/{SeService}/v{flowVersion.Major}",
            FlowType.Waarborgfonds =>
                $"{environment?.BaseAddress}/flows/waarborgfonds/{SeService}/v{flowVersion.Major}",
            FlowType.Inventarisatie =>
                $"{environment?.BaseAddress}/flows/inventarisatie/{SeService}/v{flowVersion.Major}",
            FlowType.BrEnHf => $"{environment?.BaseAddress}/flows/br_hf/{SeService}/v{flowVersion.Major}",
            FlowType.Bewindvoering =>
                $"{environment?.BaseAddress}/flows/bewindvoering/{SeService}/v{flowVersion.Major}",
            FlowType.PauzeSignaal3 =>
                $"{environment?.BaseAddress}/flows/pauze_signaal_3/{SeService}/v{flowVersion.Major}",
            _ => throw new InvalidOperationException(
                $"No service URL found for the selected flow type: {flowType} and Version: {flowVersion}")
        };

        return !string.IsNullOrEmpty(LogKey) ? url + $"?key={LogKey}" : url;
    }

    private string GetLogbookUrl(FlowType flowType, Environment environment, Version flowVersion)
    {
        var url = flowType switch
        {
            FlowType.Schuldbemiddeling => $"{environment?.BaseAddress}/{LogbookService}/v{flowVersion.Major}",
            FlowType.SbEnSk => $"{environment?.BaseAddress}/flows/sb_sk/{LogbookService}/v{flowVersion.Major}",
            FlowType.Waarborgfonds =>
                $"{environment?.BaseAddress}/flows/waarborgfonds/{LogbookService}/v{flowVersion.Major}",
            FlowType.Inventarisatie =>
                $"{environment?.BaseAddress}/flows/inventarisatie/{LogbookService}/v{flowVersion.Major}",
            FlowType.BrEnHf => $"{environment?.BaseAddress}/flows/br_hf/{LogbookService}/v{flowVersion.Major}",
            FlowType.Bewindvoering =>
                $"{environment?.BaseAddress}/flows/bewindvoering/{LogbookService}/v{flowVersion.Major}",
            FlowType.PauzeSignaal3 =>
                $"{environment?.BaseAddress}/flows/pauze_signaal_3/{LogbookService}/v{flowVersion.Major}",
            FlowType.ShvSignaal =>
                $"{environment?.BaseAddress}/flows/shv_signaal/{LogbookService}/v{flowVersion.Major}",
            _ => throw new InvalidOperationException(
                $"No service URL found for the selected flow type: {flowType} and Version: {flowVersion}")
        };

        return !string.IsNullOrEmpty(LogKey) ? url + $"?key={LogKey}" : url;
    }
}